#!/usr/bin/python

import pygtk
pygtk.require("2.0")
import gtk
import gobject
import os
import irclog

def glade_callback(function):
    """
    This decorator marks a function as a callback to a Glade signal
    Taken from http://blog.rabidgeek.com/?p=113
    """
    function.__glade_callback__ = True
    return function

class Initialiser(object):

    def __init__(self):

        builder = gtk.Builder()
        builder.add_from_file("calendar.glade")

        glade_callbacks = {}
        for key in dir(self):
            attr = getattr(self, key)
            if callable(attr) and hasattr(attr, '__glade_callback__'):
                glade_callbacks[key] = attr
        builder.connect_signals(glade_callbacks)

        self.calendar = builder.get_object("calendar1")
        self.today = self.calendar.get_date()

        self.window = builder.get_object("dialog")
        self.window.show()

    @glade_callback
    def on_ok_clicked(self, event=None):
        irclog.initialise(self.calendar.get_date(), self.today)

        self.window.hide()
        gtk.main_quit()

    @glade_callback
    def on_window_destroy(self, event=None):
        self.window.hide()
        gtk.main_quit()

class HelpFinder(object):

    def __init__(self):
        builder = gtk.Builder()
        builder.add_from_file("search.glade")

        glade_callbacks = {}
        for key in dir(self):
            attr = getattr(self, key)
            if callable(attr) and hasattr(attr, '__glade_callback__'):
                glade_callbacks[key] = attr
        builder.connect_signals(glade_callbacks)

        treeview = builder.get_object("treeview1")
        tvcolumn1 = gtk.TreeViewColumn('Question')

        treeview.append_column(tvcolumn1)

        cell = gtk.CellRendererText()
        tvcolumn1.pack_start(cell, True)
        tvcolumn1.add_attribute(cell, 'text', 1)

        self.liststore = builder.get_object("liststore1")

        self.searchbox = builder.get_object("searchbox")

        self.textbuffer = builder.get_object("textbuffer1")

        self.searchbox_timer = None

        self.window = builder.get_object("window")
        self.window.show()

    @glade_callback
    def do_delayed_search(self, searchbox, event=None):
        if self.searchbox_timer is not None:
            gobject.source_remove(self.searchbox_timer)

        self.searchbox_timer = gobject.timeout_add(500, self.do_search)

    @glade_callback
    def do_search(self, widget=None):
        if self.searchbox_timer is not None:
            gobject.source_remove(self.searchbox_timer)

        self.liststore.clear()
        if len(self.searchbox.get_text()):
            for row in irclog.get_questions(self.searchbox.get_text()):
                self.liststore.append(( row[0], row[1].split("\n")[0], ))

    @glade_callback
    def on_window_destroy(self, event=None):
        self.window.hide()
        gtk.main_quit()

    @glade_callback
    def on_treeview1_cursor_changed(self, cell):
        (path, column) = cell.get_cursor()
        if path is not None:
            self.textbuffer.set_text(irclog.get_question(self.liststore.get_value(self.liststore.get_iter(path), 0)))

if __name__ == "__main__":
    if not irclog.is_initialised():
        app = Initialiser()
        gtk.main()
    app = HelpFinder()
    gtk.main()
